@@ -6,10 +6,11 @@
 :Example:           ``session_key = khoros.auth(KhorosObject)``
 :Created By:        Jeff Shurtliff
 :Last Modified:     Jeff Shurtliff
-:Modified Date:     24 Mar 2021
+:Modified Date:     23 May 2022
 """
 
 import requests
+from defusedxml import ElementTree
 
 from . import api, errors
 from .utils import core_utils, log_utils
@@ -21,6 +22,12 @@ logger = log_utils.initialize_logging(__name__)
 def get_session_key(khoros_object, username=None, password=None):
     """This function retrieves the session key for an authentication session.
 
+    .. versionchanged:: 5.0.0
+       The error handling has been improved to provide more useful information when applicable.
+
+    .. versionchanged:: 4.2.0
+       The URI is now generated utilizing the :py:func:`khoros.auth._get_khoros_login_url` function.
+
     .. versionchanged:: 3.5.0
        This function can now be used to authenticate secondary users with either a username and password or with
        only a username if calling the function with a previously authenticated user with Administrator privileges.
@@ -43,7 +50,6 @@ def get_session_key(khoros_object, username=None, password=None):
     :raises: :py:exc:`khoros.errors.exceptions.SessionAuthenticationError`
     """
     # Prepare the API call
-    community_url = khoros_object.core_settings['community_url']
     password = khoros_object.core_settings['session_auth']['password'] if not username else password
     username = khoros_object.core_settings['session_auth']['username'] if not username else username
     payload = _get_session_key_payload(username, password)
@@ -53,7 +59,7 @@ def get_session_key(khoros_object, username=None, password=None):
     verify = api.should_verify_tls(khoros_object)
 
     # Perform the API call to authorize the session
-    uri = f"{community_url}/restapi/vc/authentication/sessions/login/?{query_string}"
+    uri = f'{_get_khoros_login_url(khoros_object)}/?{query_string}'
     secondary_user = False if password else True
     header = _get_session_key_header(khoros_object, secondary_user)
     response = requests.post(uri, headers=header, verify=verify)
@@ -65,13 +71,61 @@ def get_session_key(khoros_object, username=None, password=None):
         raise errors.exceptions.SessionAuthenticationError()
     else:
         response = response.json()
-        try:
-            session_key = response['response']['value']['$']
-        except KeyError:
-            raise errors.exceptions.SessionAuthenticationError(f"Failed to retrieve a session key for '{username}'")
+        if isinstance(response, dict) and 'response' in response and response['response'].get('status') == 'error':
+            try:
+                error_msg = response['response']['error']['message']
+                full_error_msg = f"Session key authentication failed for '{username}': {error_msg}"
+                raise errors.exceptions.SessionAuthenticationError(full_error_msg)
+            except KeyError:
+                raise errors.exceptions.SessionAuthenticationError(f"Failed to retrieve a session key for '{username}'")
+        else:
+            try:
+                session_key = response['response']['value']['$']
+            except KeyError:
+                raise errors.exceptions.SessionAuthenticationError(f"Failed to retrieve a session key for '{username}'")
     return session_key
 
 
+def get_sso_key(khoros_object):
+    """This function retrieves the session key for a LithiumSSO session.
+
+    .. versionchanged:: 5.0.0
+       The two ``if`` statements have been merged.
+
+    .. versionadded:: 4.2.0
+
+    :param khoros_object: The core Khoros object
+    :type khoros_object: class[khoros.Khoros]
+    :returns: The session key in string format
+    :raises: :py:exc:`khoros.errors.exceptions.SsoAuthenticationError`
+    """
+    khoros_login_url = _get_khoros_login_url(khoros_object)
+    headers = _get_session_key_header(khoros_object)
+
+    response = requests.post(
+        khoros_login_url,
+        headers=headers,
+        data=khoros_object.core_settings.get('sso')
+    )
+    tree = ElementTree.fromstring(response.text)
+    if 'status' in tree.attrib and tree.attrib['status'] == 'success':
+        return tree.findtext('value')
+    raise errors.exceptions.SsoAuthenticationError('Failed to retrieve a session key with the LithiumSSO token.')
+
+
+def _get_khoros_login_url(khoros_object):
+    """This function returns the URL for the Khoros login endpoint.
+
+    .. versionadded:: 4.2.0
+
+    :param khoros_object: The core Khoros object
+    :type khoros_object: class[khoros.Khoros]
+    :returns: The URL string
+    """
+    community_url = khoros_object.core_settings.get('community_url')
+    return f'{community_url}/restapi/vc/authentication/sessions/login'
+
+
 def _get_session_key_payload(_username, _password=None, _return_json=True):
     """This function constructs the payload used to request a session key.
 
