@@ -3,10 +3,10 @@
 :Module:            khoros.errors.exceptions
 :Synopsis:          Collection of exception classes relating to the khoros library
 :Usage:             ``import khoros.errors.exceptions``
-:Example:           ``raise khoros.errors.exceptions.BadCredentialsError``
+:Example:           ``raise khoros.errors.exceptions.BadCredentialsError()``
 :Created By:        Jeff Shurtliff
 :Last Modified:     Jeff Shurtliff
-:Modified Date:     22 Dec 2020
+:Modified Date:     10 Jan 2022
 """
 
 #################
@@ -83,6 +83,23 @@ class SessionAuthenticationError(KhorosError):
         super().__init__(*args)
 
 
+class SsoAuthenticationError(KhorosError):
+    """This exception is used when the SSO authentication attempt failed.
+
+    .. versionadded:: 4.2.0
+    """
+    def __init__(self, *args, **kwargs):
+        """This method defines the default or custom message for the exception."""
+        default_msg = "The SSO authentication attempt failed."
+        custom_msg = default_msg.replace('.', ' with the following message:')
+        if not (args or kwargs):
+            args = (default_msg,)
+        elif 'message' in kwargs:
+            custom_msg = f"{custom_msg} {kwargs['message']}"
+            args = (custom_msg,)
+        super().__init__(*args)
+
+
 #####################
 # General Exceptions
 #####################
@@ -91,6 +108,9 @@ class SessionAuthenticationError(KhorosError):
 class CurrentlyUnsupportedError(KhorosError):
     """This exception is used when a feature or functionality being used is currently unsupported.
 
+    .. versionchanged:: 4.5.0
+       Introduced the ability for a fully customized message to be displayed.
+
     .. versionchanged:: 2.0.0
        The unsupported feature can be passed as a string argument to explicitly reference it in the exception.
     """
@@ -99,6 +119,8 @@ class CurrentlyUnsupportedError(KhorosError):
         default_msg = "This feature is currently unsupported at this time."
         if not (args or kwargs):
             args = (default_msg,)
+        elif 'message' in kwargs:
+            args =(kwargs['message'],)
         else:
             custom_msg = f"The '{args[0]}' {default_msg.split('This ')[1]}"
             args = (custom_msg,)
@@ -163,12 +185,16 @@ class InvalidURLError(KhorosError):
 class MissingRequiredDataError(KhorosError):
     """This exception is used when a function or method is missing one or more required arguments.
 
+    .. versionchanged:: 4.0.0
+       The exception can now accept the ``param`` keyword argument.
+
     .. versionadded:: 2.0.0
     """
     def __init__(self, *args, **kwargs):
         """This method defines the default or custom message for the exception."""
         default_msg = "Missing one or more required parameters"
         init_msg = "The object failed to initialize as it is missing one or more required arguments."
+        param_msg = "The required parameter 'PARAMETER_NAME' is not defined"
         if not (args or kwargs):
             args = (default_msg,)
         elif 'init' in args or 'initialize' in args:
@@ -177,6 +203,10 @@ class MissingRequiredDataError(KhorosError):
                 args = (custom_msg,)
             else:
                 args = (init_msg,)
+        elif 'param' in kwargs:
+            args = (param_msg.replace('PARAMETER_NAME', kwargs['param']),)
+        else:
+            args = (default_msg,)
         super().__init__(*args)
 
 
@@ -212,10 +242,14 @@ class APIConnectionError(KhorosError):
 
 
 class APIRequestError(KhorosError):
-    """This exception is used for generic API request errors when there isn't a more specific exception."""
+    """This exception is used for generic API request errors when there isn't a more specific exception.
+
+    .. versionchanged:: 4.5.0
+       Fixed an issue with the default message.
+    """
     def __init__(self, *args, **kwargs):
         """This method defines the default or custom message for the exception."""
-        default_msg = "The DELETE request did not return a successful response."
+        default_msg = "The API request did not return a successful response."
         if not (args or kwargs):
             args = (default_msg,)
         super().__init__(*args)
@@ -231,6 +265,25 @@ class DELETERequestError(KhorosError):
         super().__init__(*args)
 
 
+class FeatureNotConfiguredError(KhorosError):
+    """This exception is used when an API request fails because a feature is not configured.
+
+    .. versionadded:: 4.0.0
+    """
+    def __init__(self, *args, **kwargs):
+        """This method defines the default or custom message for the exception."""
+        exc_msg = "The feature is not configured."
+        if 'identifier' in kwargs or 'feature' in kwargs:
+            if 'identifier' in kwargs:
+                exc_msg += f" Identifier: {kwargs['identifier']}"
+            if 'feature' in kwargs:
+                exc_msg = exc_msg.replace("feature", f"{kwargs['feature']} feature")
+            args = (exc_msg,)
+        elif not (args or kwargs):
+            args = (exc_msg,)
+        super().__init__(*args)
+
+
 class GETRequestError(KhorosError):
     """This exception is used for generic GET request errors when there isn't a more specific exception.
 
@@ -490,6 +543,22 @@ class TooManyResultsError(KhorosError):
 #####################
 
 
+class InvalidMetadataError(KhorosError):
+    """This exception is used when there is an issue involving message metadata.
+
+    .. versionadded:: 4.5.0
+    """
+    def __init__(self, *args, **kwargs):
+        """This method defines the default or custom message for the exception."""
+        default_msg = "The message metadata is invalid."
+        if not (args or kwargs):
+            args = (default_msg,)
+        elif 'metadata' in kwargs:
+            custom_msg = f"{default_msg.split('is')[0]}'{kwargs.get('metadata')}'{default_msg.split('is')[1]}"
+            args = (custom_msg,)
+        super().__init__(*args)
+
+
 class MessageTypeNotFoundError(KhorosError):
     """This exception is used when a message type cannot be identified from a given URL."""
     def __init__(self, *args, **kwargs):
@@ -506,6 +575,16 @@ class MessageTypeNotFoundError(KhorosError):
         super().__init__(*args)
 
 
+class InvalidMessagePayloadError(KhorosError):
+    """This exception is used when the payload for creating a message is invalid."""
+    def __init__(self, *args, **kwargs):
+        """This method defines the default or custom message for the exception."""
+        default_msg = "The message payload is invalid and cannot be utilized."
+        if not (args or kwargs):
+            args = (default_msg,)
+        super().__init__(*args)
+
+
 ##################
 # Node Exceptions
 ##################
