@@ -7,16 +7,17 @@
                     node_id='support-tkb')``
 :Created By:        Jeff Shurtliff
 :Last Modified:     Jeff Shurtliff
-:Modified Date:     10 Mar 2021
+:Modified Date:     26 Sep 2022
 """
 
+import json
 import warnings
 
 from . import attachments, users
 from . import tags as tags_module
 from .. import api, liql, errors
 from ..structures import nodes
-from ..utils import log_utils
+from ..utils import log_utils, core_utils
 
 # Initialize the logger for this module
 logger = log_utils.initialize_logging(__name__)
@@ -40,10 +41,21 @@ def create(khoros_object, subject=None, body=None, node=None, node_id=None, node
            context_id=None, context_url=None, cover_image=None, images=None, is_answer=None, is_draft=None,
            labels=None, product_category=None, products=None, read_only=None, seo_title=None, seo_description=None,
            tags=None, ignore_non_string_tags=False, teaser=None, topic=None, videos=None, attachment_file_paths=None,
-           full_response=False, return_id=False, return_url=False, return_api_url=False, return_http_code=False,
-           return_status=None, return_error_messages=None, split_errors=False):
+           full_payload=None, full_response=False, return_id=False, return_url=False, return_api_url=False,
+           return_http_code=False, return_status=None, return_error_messages=None, split_errors=False,
+           proxy_user_object=None):
     """This function creates a new message within a given node.
 
+    .. versionchanged:: 4.5.0
+       The Content-Type header is now explicitly defined as ``application/json`` when handling non-multipart requests.
+
+    .. versionchanged:: 4.4.0
+       Introduced the ``proxy_user_object`` parameter to allow messages to be created on behalf of other users.
+
+    .. versionchanged:: 4.3.0
+       It is now possible to pass the pre-constructed full JSON payload into the function via the ``full_payload``
+       parameter as an alternative to defining each field individually.
+
     .. versionchanged:: 2.8.0
        The ``ignore_non_string_tags``, ``return_status``, ``return_error_messages`` and ``split_errors``
        arguments were introduced.
@@ -104,6 +116,21 @@ def create(khoros_object, subject=None, body=None, node=None, node_id=None, node
     :type videos: dict, None
     :param attachment_file_paths: The full path(s) to one or more attachment (e.g. ``path/to/file1.pdf``)
     :type attachment_file_paths: str, tuple, list, set, None
+    :param full_payload: Pre-constructed full JSON payload as a dictionary (*preferred*) or a JSON string with the
+                         following syntax:
+
+                            .. code-block:: json
+
+                               {
+                                 "data": {
+                                   "type": "message",
+
+                                 }
+                               }
+
+                         .. note:: The ``type`` field shown above is essential for the payload to be valid.
+
+    :type full_payload: dict, str, None
     :param full_response: Defines if the full response should be returned instead of the outcome (``False`` by default)
 
                           .. caution:: This argument overwrites the ``return_id``, ``return_url``, ``return_api_url``
@@ -126,23 +153,65 @@ def create(khoros_object, subject=None, body=None, node=None, node_id=None, node
     :type return_error_messages: bool, None
     :param split_errors: Defines whether or not error messages should be merged when applicable
     :type split_errors: bool
+    :param proxy_user_object: Instantiated :py:class:`khoros.objects.users.ImpersonatedUser` object to create the
+                              message on behalf of a secondary user.
+    :type proxy_user_object: class[khoros.objects.users.ImpersonatedUser], None
     :returns: Boolean value indicating a successful outcome (default) or the full API response
     :raises: :py:exc:`TypeError`, :py:exc:`ValueError`, :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`,
              :py:exc:`khoros.errors.exceptions.DataMismatchError`
     """
     api_url = f"{khoros_object.core['v2_base']}/messages"
-    payload = construct_payload(subject, body, node, node_id, node_url, canonical_url, context_id, context_url,
-                                is_answer, is_draft, read_only, seo_title, seo_description, teaser, tags, cover_image,
-                                images, labels, product_category, products, topic, videos,
-                                ignore_non_string_tags=ignore_non_string_tags, khoros_object=khoros_object)
+    if full_payload:
+        payload = validate_message_payload(full_payload)
+    else:
+        payload = construct_payload(subject, body, node, node_id, node_url, canonical_url, context_id, context_url,
+                                    is_answer, is_draft, read_only, seo_title, seo_description, teaser, tags,
+                                    cover_image, images, labels, product_category, products, topic, videos,
+                                    ignore_non_string_tags=ignore_non_string_tags, khoros_object=khoros_object)
+        payload = validate_message_payload(payload)
     multipart = True if attachment_file_paths else False
     if multipart:
         payload = attachments.construct_multipart_payload(payload, attachment_file_paths)
-    response = api.post_request_with_retries(api_url, payload, khoros_object=khoros_object, multipart=multipart)
+    content_type = 'application/json' if not multipart else None
+    response = api.post_request_with_retries(api_url, payload, khoros_object=khoros_object, multipart=multipart,
+                                             content_type=content_type, proxy_user_object=proxy_user_object)
     return api.deliver_v2_results(response, full_response, return_id, return_url, return_api_url, return_http_code,
                                   return_status, return_error_messages, split_errors, khoros_object)
 
 
+def validate_message_payload(payload):
+    """This function validates the payload for a message to ensure that it can be successfully utilized.
+
+    .. versionadded:: 4.3.0
+
+    :param payload: The message payload to be validated as a dictionary (*preferred*) or a JSON string.
+    :type payload: dict, str
+    :returns: The payload as a dictionary
+    :raises: :py:exc:`khoros.errors.exceptions.InvalidMessagePayloadError`
+    """
+    if not payload and not isinstance(payload, dict) and not isinstance(payload, str):
+        raise errors.exceptions.InvalidMessagePayloadError("The message payload is null.")
+    if isinstance(payload, str):
+        logger.warning("The message payload is defined as a JSON string and will be converted to a dictionary.")
+        payload = json.loads(payload)
+    if not isinstance(payload, dict):
+        raise errors.exceptions.InvalidMessagePayloadError("The message payload must be a dictionary or "
+                                                           "JSON string.")
+    if 'data' not in payload:
+        raise errors.exceptions.InvalidMessagePayloadError("The message payload must include the 'data' key.")
+    if 'type' not in payload.get('data'):
+        raise errors.exceptions.InvalidMessagePayloadError("The message payload must include the `type` key (with "
+                                                           "'message' as the value) within the 'data' parent key.")
+    if payload.get('data').get('type') != 'message':
+        raise errors.exceptions.InvalidMessagePayloadError("The value for the 'type' key in the message payload "
+                                                           "must be defined  as 'message' but was defined as "
+                                                           f"'{payload.get('data').get('type')}' instead.")
+    if 'subject' not in payload.get('data' or 'board' not in payload.get('data') or
+                                    'id' not in payload.get('data').get('board')):
+        raise errors.exceptions.InvalidMessagePayloadError("A node and subject must be defined.")
+    return payload
+
+
 def construct_payload(subject=None, body=None, node=None, node_id=None, node_url=None, canonical_url=None,
                       context_id=None, context_url=None, is_answer=None, is_draft=None, read_only=None, seo_title=None,
                       seo_description=None, teaser=None, tags=None, cover_image=None, images=None, labels=None,
@@ -151,6 +220,11 @@ def construct_payload(subject=None, body=None, node=None, node_id=None, node_url
                       ignore_non_string_tags=False, msg_id=None, khoros_object=None, action='create'):
     """This function constructs and properly formats the JSON payload for a messages API request.
 
+    .. todo::
+       Add support for the following parameters which are currently present but unsupported: ``cover_image``,
+       ``images``, ``labels``, ``product_category``, ``products``, ``topic``, ``videos``, ``parent``, ``status``,
+       ``attachments_to_add`` and ``attachments to remove``
+
     .. versionchanged:: 2.8.0
        Added the ``parent``, ``status``, ``moderation_status``, ``attachments_to_add``, ``attachments_to_remove``,
        ``overwrite_tags``, ``ignore_non_string_tags``, ``msg_id``, ``khoros_object`` and ``action`` arguments, and
@@ -311,9 +385,12 @@ def update(khoros_object, msg_id=None, msg_url=None, subject=None, body=None, no
            status=None, seo_title=None, seo_description=None, tags=None, overwrite_tags=False,
            ignore_non_string_tags=False, teaser=None, attachments_to_add=None, attachments_to_remove=None,
            full_response=None, return_id=None, return_url=None, return_api_url=None, return_http_code=None,
-           return_status=None, return_error_messages=None, split_errors=False):
+           return_status=None, return_error_messages=None, split_errors=False, proxy_user_object=None):
     """This function updates one or more elements of an existing message.
 
+    .. versionchanged:: 4.4.0
+       Introduced the ``proxy_user_object`` parameter to allow messages to be updated on behalf of other users.
+
     .. versionadded:: 2.8.0
 
     :param khoros_object: The core :py:class:`khoros.Khoros` object
@@ -415,6 +492,9 @@ def update(khoros_object, msg_id=None, msg_url=None, subject=None, body=None, no
     :type return_error_messages: bool, None
     :param split_errors: Defines whether or not error messages should be merged when applicable
     :type split_errors: bool
+    :param proxy_user_object: Instantiated :py:class:`khoros.objects.users.ImpersonatedUser` object to update the
+                              message on behalf of a secondary user.
+    :type proxy_user_object: class[khoros.objects.users.ImpersonatedUser], None
     :returns: Boolean value indicating a successful outcome (default) or the full API response
     :raises: :py:exc:`TypeError`, :py:exc:`ValueError`, :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`,
              :py:exc:`khoros.errors.exceptions.DataMismatchError`
@@ -431,11 +511,192 @@ def update(khoros_object, msg_id=None, msg_url=None, subject=None, body=None, no
     multipart = True if attachments_to_add else False
     if multipart:
         payload = attachments.construct_multipart_payload(payload, attachments_to_add, 'update')
-    response = api.put_request_with_retries(api_url, payload, khoros_object=khoros_object, multipart=multipart)
+    response = api.put_request_with_retries(api_url, payload, khoros_object=khoros_object, multipart=multipart,
+                                            proxy_user_object=proxy_user_object)
     return api.deliver_v2_results(response, full_response, return_id, return_url, return_api_url, return_http_code,
                                   return_status, return_error_messages, split_errors, khoros_object)
 
 
+def kudo(khoros_object, msg_id):
+    """This function kudos (i.e. "likes") a message.
+
+    .. versionadded:: 5.1.0
+
+    :param khoros_object: The core :py:class:`khoros.Khoros` object
+    :type khoros_object: class[khoros.Khoros]
+    :param msg_id: The ID of the message to be kudoed
+    :type msg_id: str, int
+    :returns: The API response in JSON format
+    :raises: :py:exc:`khoros.errors.exceptions.APIConnectionError`,
+             :py:exc:`khoros.errors.exceptions.POSTRequestError`
+    """
+    # Define the payload
+    payload = {
+        "data": {
+            "type": "kudo"
+        }
+    }
+
+    # Define the API endpoint URI
+    uri = f'{khoros_object.core["v2_base"]}/messages/{msg_id}/kudos'
+
+    # Perform the API call
+    return api.post_request_with_retries(uri, payload, khoros_object=khoros_object)
+
+
+def _set_spam(_khoros_object, _msg_id, _action='spam'):
+    """This function flags a message as ``spam`` or ``not_spam`` as a moderation action.
+
+    .. versionadded:: 5.1.0
+
+    :param _khoros_object: The core :py:class:`khoros.Khoros` object
+    :type _khoros_object: class[khoros.Khoros]
+    :param _msg_id: The ID of the message to be moderated
+    :type _msg_id: str, int
+    :param _action: Defines the message as ``spam`` (default) or ``not_spam``
+    :type _action: str
+    :returns: The API response in JSON format
+    :raises: :py:exc:`khoros.errors.exceptions.APIConnectionError`,
+             :py:exc:`khoros.errors.exceptions.POSTRequestError`
+    """
+    # Define the payload
+    payload = {
+        "data": {
+            "type": "moderation_data",
+            "action": f"{_action}"
+        }
+    }
+
+    # Define the API endpoint URI
+    uri = f'{_khoros_object.core["v2_base"]}/messages/{_msg_id}/moderation_data'
+
+    # Perform the API call
+    return api.put_request_with_retries(uri, payload, khoros_object=_khoros_object)
+
+
+def flag(khoros_object, msg_id):
+    """This function flags a message as spam.
+
+    .. versionadded:: 5.1.0
+
+    :param khoros_object: The core :py:class:`khoros.Khoros` object
+    :type khoros_object: class[khoros.Khoros]
+    :param msg_id: The ID of the message to be flagged
+    :type msg_id: str, int
+    :returns: The API response in JSON format
+    :raises: :py:exc:`khoros.errors.exceptions.APIConnectionError`,
+             :py:exc:`khoros.errors.exceptions.POSTRequestError`
+    """
+    return _set_spam(khoros_object, msg_id, 'spam')
+
+
+def unflag(khoros_object, msg_id):
+    """This function flags a message as not being spam.
+
+    .. versionadded:: 5.1.0
+
+    :param khoros_object: The core :py:class:`khoros.Khoros` object
+    :type khoros_object: class[khoros.Khoros]
+    :param msg_id: The ID of the message to be flagged
+    :type msg_id: str, int
+    :returns: The API response in JSON format
+    :raises: :py:exc:`khoros.errors.exceptions.APIConnectionError`,
+             :py:exc:`khoros.errors.exceptions.POSTRequestError`
+    """
+    return _set_spam(khoros_object, msg_id, 'not_spam')
+
+
+def label(khoros_object, msg_id, label_text):
+    """This function adds a single label to a given message.
+
+    .. versionadded:: 5.1.0
+
+    :param khoros_object: The core :py:class:`khoros.Khoros` object
+    :type khoros_object: class[khoros.Khoros]
+    :param msg_id: The ID of the message to be flagged
+    :type msg_id: str, int
+    :param label_text: The label to be added
+    :type label_text: str
+    :returns: The API response in JSON format
+    :raises: :py:exc:`khoros.errors.exceptions.APIConnectionError`,
+             :py:exc:`khoros.errors.exceptions.POSTRequestError`
+    """
+    # Define the payload
+    payload = {
+        "data": {
+            "type": "label",
+            "text": f"{label_text}"
+        }
+    }
+
+    # Define the API endpoint URI
+    uri = f'{khoros_object.core["v2_base"]}/messages/{msg_id}/labels'
+
+    # Perform the API call
+    return api.post_request_with_retries(uri, payload, khoros_object=khoros_object)
+
+
+def tag(khoros_object, msg_id, tag_text):
+    """This function adds a single tag to a given message.
+
+    .. versionadded:: 5.1.0
+
+    :param khoros_object: The core :py:class:`khoros.Khoros` object
+    :type khoros_object: class[khoros.Khoros]
+    :param msg_id: The ID of the message to be flagged
+    :type msg_id: str, int
+    :param tag_text: The tag to be added
+    :type tag_text: str
+    :returns: The API response in JSON format
+    :raises: :py:exc:`khoros.errors.exceptions.APIConnectionError`,
+             :py:exc:`khoros.errors.exceptions.POSTRequestError`
+    """
+    # Define the payload
+    payload = {
+        "data": {
+            "type": "tag",
+            "text": f"{tag_text}"
+        }
+    }
+
+    # Define the API endpoint URI
+    uri = f'{khoros_object.core["v2_base"]}/messages/{msg_id}/tags'
+
+    # Perform the API call
+    return api.post_request_with_retries(uri, payload, khoros_object=khoros_object)
+
+
+def get_metadata(khoros_object, msg_id, metadata_key):
+    """This function retrieves the value for a specific metadata key associated with a given message.
+
+    .. versionadded:: 4.5.0
+
+    :param khoros_object: The core :py:class:`khoros.Khoros` object
+    :type khoros_object: class[khoros.Khoros]
+    :param msg_id: The ID of the message for which the metadata will be retrieved
+    :type msg_id: str, int
+    :param metadata_key: The metadata key for which the value will be retrieved
+    :type metadata_key: str
+    :returns: The metadata value
+    :raises: :py:exc:`khoros.errors.exceptions.MissingRequiredDataError',
+             :py:exc:`khoros.errors.exceptions.InvalidMetadataError`,
+             :py:exc:`khoros.errors.exceptions.GETRequestError`
+    """
+    if not msg_id or not metadata_key:
+        raise errors.exceptions.MissingRequiredDataError('A message ID and a metadata key are required')
+    uri = f'/messages/id/{msg_id}/metadata/key/{metadata_key}'
+    response = khoros_object.v1.get(uri)
+    if not response.get('response'):
+        raise errors.exceptions.GETRequestError('The GET request to retrieve the message metadata was not successful')
+    if response['response'].get('status') == 'error':
+        if response['response']['error'].get('message'):
+            raise errors.exceptions.InvalidMetadataError(response['response']['error'].get('message'))
+        else:
+            raise errors.exceptions.InvalidMetadataError()
+    metadata_value = response['response']['value'].get('$')
+    return metadata_value
+
+
 def _verify_message_id(_msg_id, _msg_url):
     """This function verifies that a message ID has been defined or can be using the message URL.
 
@@ -450,7 +711,7 @@ def _verify_message_id(_msg_id, _msg_url):
              :py:exc:`errors.exceptions.MessageTypeNotFoundError`
     """
     if not any((_msg_id, _msg_url)):
-        raise errors.exceptions.MissingRequiredDataError("A message ID or URL must be defined when updating messages")
+        raise errors.exceptions.MissingRequiredDataError('A message ID or URL must be defined when updating messages')
     elif not _msg_id:
         _msg_id = get_id_from_url(_msg_url)
     return _msg_id
@@ -853,3 +1114,111 @@ def format_content_mention(khoros_object=None, content_info=None, content_id=Non
         url = f"{khoros_object.core['base_url']}{url}"
     mention_tag = f'<li-message title="{title}" uid="{content_id}" url="{url}"></li-message>'
     return mention_tag
+
+
+def get_context_id(khoros_object, msg_id):
+    """This function retrieves the Context ID value for a given message ID.
+
+    .. versionadded:: 5.0.0
+
+    :param khoros_object: The core :py:class:`khoros.Khoros` object
+    :type khoros_object: class[khoros.Khoros]
+    :param msg_id: The message ID to query
+    :type msg_id: str
+    :returns: The value of the Context ID metadata field
+    :raises: :py:exc:`khoros.errors.exceptions.get_context_id`
+    """
+    try:
+        query = f"SELECT context_id FROM messages WHERE id = '{msg_id}'"
+        response = khoros_object.query(query, return_items=True)[0]
+        if isinstance(response, dict) and 'context_id' in response:
+            context_id = response.get('context_id')
+        else:
+            context_id = ''
+    except Exception as exc:
+        raise errors.exceptions.InvalidMetadataError('Encountered the following exception while retrieving the '
+                                                     f'context_id value: {exc}')
+    return context_id
+
+
+def get_context_url(khoros_object, msg_id):
+    """This function retrieves the Context URL value for a given message ID.
+
+    .. versionadded:: 5.0.0
+
+    :param khoros_object: The core :py:class:`khoros.Khoros` object
+    :type khoros_object: class[khoros.Khoros]
+    :param msg_id: The message ID to query
+    :type msg_id: str
+    :returns: The value of the Context URL metadata field
+    :raises: :py:exc:`khoros.errors.exceptions.get_context_id`
+    """
+    try:
+        query = f"SELECT context_url FROM messages WHERE id = '{msg_id}'"
+        response = khoros_object.query(query, return_items=True)[0]
+        if isinstance(response, dict) and 'context_url' in response:
+            context_url = response.get('context_url')
+        else:
+            context_url = ''
+    except Exception as exc:
+        raise errors.exceptions.InvalidMetadataError('Encountered the following exception while retrieving the '
+                                                     f'context_url value: {exc}')
+    return context_url
+
+
+def define_context_id(khoros_object, msg_id, context_id='', full_response=False):
+    """This function defines the context_id metadata value for a given message.
+
+    .. versionadded:: 5.0.0
+
+    :param khoros_object: The core :py:class:`khoros.Khoros` object
+    :type khoros_object: class[khoros.Khoros]
+    :param msg_id: The message ID to query
+    :type msg_id: str
+    :param context_id: The value to be written to the context_id metadata field (Empty by default)
+    :type context_id: str
+    :param full_response: Determines if the full API response should be returned (``False`` by default)
+    :type full_response: bool
+    :returns: A Boolean value to indicate the success of the operation or alternatively the full API response
+    :raises: :py:exc:`khoros.errors.exceptions.APIRequestError`
+    """
+    successful = False
+    context_id = core_utils.url_encode(context_id)
+    path = f'/messages/id/{msg_id}/metadata/key/message.context_id/set?value={context_id}'
+    try:
+        response = api.make_v1_request(khoros_object, path, request_type='POST')
+        if isinstance(response, dict) and 'response' in response and response['response'].get('status') == 'success':
+            successful = True
+    except Exception as exc:
+        raise errors.exceptions.APIRequestError('Encountered the following exception while defining the context_id '
+                                                f'value: {exc}')
+    return response if full_response else successful
+
+
+def define_context_url(khoros_object, msg_id, context_url='', full_response=False):
+    """This function defines the context_url metadata value for a given message.
+
+    .. versionadded:: 5.0.0
+
+    :param khoros_object: The core :py:class:`khoros.Khoros` object
+    :type khoros_object: class[khoros.Khoros]
+    :param msg_id: The message ID to query
+    :type msg_id: str
+    :param context_url: The value to be written to the context_url metadata field (Empty by default)
+    :type context_url: str
+    :param full_response: Determines if the full API response should be returned (``False`` by default)
+    :type full_response: bool
+    :returns: A Boolean value to indicate the success of the operation or alternatively the full API response
+    :raises: :py:exc:`khoros.errors.exceptions.APIRequestError`
+    """
+    successful = False
+    context_url = core_utils.url_encode(context_url)
+    path = f'/messages/id/{msg_id}/metadata/key/message.context_url/set?value={context_url}'
+    try:
+        response = api.make_v1_request(khoros_object, path, request_type='POST')
+        if isinstance(response, dict) and 'response' in response and response['response'].get('status') == 'success':
+            successful = True
+    except Exception as exc:
+        raise errors.exceptions.APIRequestError('Encountered the following exception while defining the context_url '
+                                                f'value: {exc}')
+    return response if full_response else successful
