@@ -6,16 +6,17 @@
 :Example:           ``exceptions = resources.import_exceptions_module()``
 :Created By:        Jeff Shurtliff
 :Last Modified:     Jeff Shurtliff
-:Modified Date:     28 Jun 2021
+:Modified Date:     27 Sep 2022
 """
 
 import os
 import sys
+import json
 import importlib
 
 import yaml
 
-# Define global variable to store the YAML test ettings
+# Define global variable to store the YAML test settings
 test_config = {}
 
 
@@ -24,10 +25,12 @@ def set_package_path():
 
     .. versionadded:: 2.7.4
 
+    .. versionchanged:: 5.0.0
+       Removed the redundant return statement.
+
     :returns: None
     """
     sys.path.insert(0, os.path.abspath('../..'))
-    return
 
 
 def import_modules(*modules):
@@ -45,18 +48,91 @@ def import_modules(*modules):
     return imported_modules if len(imported_modules) > 1 else imported_modules[0]
 
 
-def initialize_khoros_object():
+def _get_control_dataset_file(_dataset_name):
+    """This function returns the file name associated with a particular dataset.
+
+    .. versionadded:: 5.1.0
+
+    :param _dataset_name: The name of the dataset
+    :type _dataset_name: str
+    :returns: The file name if found or a blank string
+    """
+    # Define the dataset file mapping
+    _datasets = {
+        'categories': 'categories_control_data.json',
+        'communities': 'communities_control_data.json',
+    }
+    try:
+        _dataset_file = _datasets[_dataset_name]
+    except KeyError:
+        _dataset_file = ''
+    return _dataset_file
+
+
+def import_control_data(dataset_name):
+    """This function imports a local control data file as a dictionary.
+
+    .. versionadded:: 5.1.0
+
+    :param dataset_name: The name of the dataset
+    :type dataset_name: str
+    :returns: The JSON data as a dictionary
+    """
+    dataset_file = _get_control_dataset_file(dataset_name)
+    with open(f'local/tests/{dataset_file}', 'r') as file:
+        data = json.load(file)
+    return data
+
+
+def control_data_exists(dataset_name):
+    """This function checks to see if a local control data file exists.
+
+    .. versionadded:: 5.1.0
+
+    :param dataset_name: The name of the dataset
+    :type dataset_name: str
+    :returns: Boolean value indicating whether the file was found
+    """
+    data_exists = False
+    dataset_file = _get_control_dataset_file(dataset_name)
+    if dataset_file and os.path.isfile(f'local/tests/{dataset_file}'):
+        data_exists = True
+    return data_exists
+
+
+def initialize_khoros_object(use_defined_settings=False, defined_settings=None, append_to_default=False):
     """This function imports the :py:class:`khoros.core.Khoros` class and initializes an object.
 
+    .. versionchanged:: 4.3.0
+       Added support for utilizing the ``defined_settings`` parameter.
+
     .. versionadded:: 2.7.4
 
     :returns: The initialized :py:class:`khoros.core.Khoros` object
     """
     set_package_path()
+    default_defined_settings = {
+        'community_url': 'https://community.example.com',
+        'auto_connect': False,
+        'tenant_id': 'example',
+        'auth_type': 'session_auth',
+        'session_auth': {
+            'username': 'testuser',
+            'password': 'fakePassword123',
+        },
+    }
     core_module = importlib.import_module('khoros.core')
-    return core_module.Khoros(community_url='https://example.community.com', auto_connect=False,
-                              tenant_id='example', auth_type='session_auth',
-                              session_auth={'username': 'testuser', 'password': 'fakePassword123'})
+    if use_defined_settings:
+        settings = default_defined_settings if defined_settings is None else defined_settings
+        if defined_settings and append_to_default:
+            settings = default_defined_settings
+            settings.update(defined_settings)
+        instantiated_object = core_module.Khoros(defined_settings=settings)
+    else:
+        instantiated_object = core_module.Khoros(community_url='https://community.example.com', auto_connect=False,
+                                                 tenant_id='example', auth_type='session_auth',
+                                                 session_auth={'username': 'testuser', 'password': 'fakePassword123'})
+    return instantiated_object
 
 
 def get_structure_collection(structure_type):
@@ -85,7 +161,7 @@ def _get_local_helper_file_name(_production=False):
 
     .. versionadded:: 4.1.0
 
-    :param _production: Defines whether or not the helper file is associated with a Production environment
+    :param _production: Defines whether the helper file is associated with a Production environment
     :type _production: bool, None
     :returns: The file name for the local helper file
     """
@@ -101,9 +177,9 @@ def local_helper_exists(production=False):
 
     .. versionadded:: 4.1.0
 
-    :param production: Defines whether or not the helper file is associated with a Production environment
+    :param production: Defines whether the helper file is associated with a Production environment
     :type production: bool, None
-    :returns: Boolean value indicating whether or not the local helper file was found
+    :returns: Boolean value indicating whether the local helper file was found
     """
     file_name = _get_local_helper_file_name(production)
     return os.path.exists(f'local/{file_name}')
@@ -114,7 +190,7 @@ def local_test_config_exists():
 
     .. versionadded:: 4.1.0
 
-    :returns: Boolean value indicating whether or not the file was found
+    :returns: Boolean value indicating whether the file was found
     """
     return os.path.exists('local/khorostest.yml')
 
@@ -124,13 +200,15 @@ def parse_testing_config_file():
 
     .. versionadded:: 4.1.0
 
+    .. versionchanged:: 5.0.0
+       Removed the redundant return statement.
+
     :returns: None
     """
     global test_config
     if local_test_config_exists():
         with open('local/khorostest.yml', 'r') as file:
             test_config = yaml.safe_load(file)
-    return
 
 
 def get_testing_config():
@@ -148,7 +226,7 @@ def instantiate_with_local_helper(production=False):
 
     .. versionadded:: 4.1.0
 
-    :param production: Defines whether or not the helper file is associated with a Production environment
+    :param production: Defines whether the helper file is associated with a Production environment
     :type production: bool, None
     :returns: The instantiated :py:class:`khoros.core.Khoros` object
     """
@@ -157,3 +235,15 @@ def instantiate_with_local_helper(production=False):
         set_package_path()
         core_module = importlib.import_module('khoros.core')
         return core_module.Khoros(helper=f"local/{file_name}")
+
+
+def instantiate_with_placeholder():
+    """This function instantiates a Khoros object with placeholder data.
+
+    .. versionadded:: 5.0.0
+
+    :returns: The instantiated :py:class:`khoros.core.Khoros` object
+    """
+    set_package_path()
+    core_module = importlib.import_module('khoros.core')
+    return core_module.Khoros(placeholder=True)
